import MainGame.AIPlayer;
import MainGame.Game;
import MainGame.Player;
import MainGame.Tile;
import Servers.Server;

import java.util.*;
/**
 * The {@code Main} class is the entry point for the Rummikub game.
 * It initializes the server, manages players (both human and AI),
 * and controls the game loop until a player reaches the winning score.
 */
public class Main {
    private static final int MAX_PLAYERS = 4; // Limit to 4 players including AI
    private static final int WINNING_SCORE = 200; // First to 200 points wins the game
    /**
     * The main method that starts the game.
     *
     * @param args Command-line arguments (not used).
     */
    public static void main(String[] args) {
        Scanner scanner = new Scanner(System.in);
        List<Player> players = new ArrayList<>();
        Map<Player, Integer> scores = new HashMap<>(); // Store cumulative scores for multiple rounds

        // ✅ **Step 1: Start the Server FIRST**
        Game game = new Game(players);
        Server server = new Server(5000, game);

        new Thread(() -> server.startServer()).start();
        System.out.println("✅ Game server started. Waiting for human players to join...");

        // ✅ **Step 2: Wait for at least 2 human players to join**
        while (players.size() < 2) {
            System.out.println("⌛ Waiting for at least 2 human players to join... (" + players.size() + " joined)");
            try {
                Thread.sleep(2000);
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                break;
            }
        }

        // ✅ **Step 3: Ask if AI players should be added AFTER 2 players join**
        System.out.println("Do you want to add AI players? (yes/no): ");
        String addAI = scanner.nextLine().trim().toLowerCase();

        int aiPlayers = 0;
        if (addAI.equals("yes")) {
            while (true) {
                System.out.println("Enter number of AI players to add (Max " + (MAX_PLAYERS - players.size()) + "): ");
                try {
                    aiPlayers = Integer.parseInt(scanner.nextLine().trim());
                    if (aiPlayers >= 0 && aiPlayers <= (MAX_PLAYERS - players.size())) break;
                    System.out.println("Invalid number. Please enter a value between 0 and " + (MAX_PLAYERS - players.size()));
                } catch (NumberFormatException e) {
                    System.out.println("Please enter a valid number.");
                }
            }
        }

        // ✅ **Step 4: Add AI players AFTER human players are confirmed**
        for (int i = 0; i < aiPlayers; i++) {
            AIPlayer aiPlayer = new AIPlayer("AI_Player_" + (i + 1));
            players.add(aiPlayer);
            System.out.println("🤖 Added AI Player: " + aiPlayer.getName());
        }

        // ✅ **Step 5: Initialize Scores for ALL Players (Including AI)**
        for (Player player : players) {
            scores.put(player, 0); // Ensure AI and humans are in score tracking
        }

        // ✅ **Step 6: Start the Game AFTER AI & Human Players Are Set**
        game.initializeGame();
        game.startGame();
        displayGame(game);

        while (true) {
            // ✅ **Step 7: Update Scores for ALL Players**
            Map<Player, Integer> roundScores = game.calculateScores();
            for (Player player : players) {
                scores.put(player, scores.get(player) + roundScores.getOrDefault(player, 0));
            }

            System.out.println("\n📊 Round Over! Current Scores:");
            for (Map.Entry<Player, Integer> entry : scores.entrySet()) {
                System.out.println(entry.getKey().getName() + ": " + entry.getValue() + " points");
            }

            // ✅ **Step 8: Check for Game Winner**
            for (Map.Entry<Player, Integer> entry : scores.entrySet()) {
                if (entry.getValue() >= WINNING_SCORE) {
                    System.out.println("\n🎉 Game Over! Winner: " + entry.getKey().getName() + " 🎉");
                    System.out.println("\n🏆 Final Scores 🏆");
                    for (Map.Entry<Player, Integer> finalScore : scores.entrySet()) {
                        System.out.println(finalScore.getKey().getName() + ": " + finalScore.getValue() + " points");
                    }
                    return;
                }
            }

            System.out.println("\n🔄 Starting a new round...\n");
        }
    }
    /**
     * Displays the current game state, including the board, players' hands, and the current turn.
     *
     * AI players automatically make moves, while human players are prompted for input.
     *
     * @param game The current game instance.
     */
    private static void displayGame(Game game) {
        while (!game.checkWinner()) {
            Player currentPlayer = game.getCurrentPlayer();
            System.out.println("\n🃏 Current Board:");
            for (List<Tile> set : game.getBoard().getSets()) {
                System.out.println(set);
            }
            System.out.println("🎯 TURN: " + currentPlayer.getName());
            System.out.println("🃏 Player Hand: " + currentPlayer.getHand()); // ✅ Show AI & Human hands

            if (currentPlayer instanceof AIPlayer) {
                // ✅ **Show AI's Hand Before It Moves**
                System.out.println("🤖 AI's Current Hand: " + currentPlayer.getHand());

                // ✅ **AI Automatically Makes a Move**
                System.out.println("🤖 AI is thinking...");
                String aiMove = ((AIPlayer) currentPlayer).makeMove(game);
                System.out.println("🤖 AI Move: " + aiMove);

                String response = game.processMove(aiMove, currentPlayer);
                System.out.println(response);

                // ✅ **Ensure AI Move is Processed Correctly**
                if (!response.startsWith("INVALID")) {
                    game.nextTurn(); // ✅ Move to next player only if the move is valid
                } else {
                    System.out.println("🤖 AI move was invalid. Retrying...");
                }
            } else {
                // ✅ **Human Players Still Get Input**
                Scanner scanner = new Scanner(System.in);
                String response;
                do {
                    System.out.println("Enter move (P <tile>, M <tile> <source_set> <target_set>, S <source_set> <tile1> <tile2>, J <set1> <set2>, DRAW, EXIT):");
                    String move = scanner.nextLine();
                    response = game.processMove(move, currentPlayer);
                    System.out.println(response);
                } while (response.startsWith("INVALID")); // Keep asking until a valid move

                game.nextTurn(); // ✅ Move to next player after human turn
            }
        }
    }


}



