package MainGame;

import java.util.*;

/**
 * The {@code AIPlayer} class represents an AI-controlled player in the game.
 * It extends the {@code Player} class and implements logic to make strategic moves.
 */
public class AIPlayer extends Player {
    /**
     * Constructs an {@code AIPlayer} with the specified name.
     *
     * @param name The name of the AI player.
     */
    public AIPlayer(String name) {
        super(name);
    }

    /**
     * Determines the best possible move for the AI player based on the current game state.
     *
     * @param game The current game instance.
     * @return A string representing the chosen move command.
     */
    public String makeMove(Game game) {
        List<Tile> hand = getHand();

        // **Step 1: Check if AI can extend an existing set**
        for (int i = 0; i < game.getBoard().getSets().size(); i++) {
            List<Tile> set = new ArrayList<>(game.getBoard().getSets().get(i));
            for (Tile tile : hand) {
                set.add(tile);
                if (game.isValidMeld(set)) {
                    return "M " + tile.getNumber() + " " + tile.getColor() + " " + i; // Move tile to set
                }
                set.remove(tile); // Undo move if invalid
            }
        }

        // **Step 2: Try to play a valid meld**
        List<List<Tile>> possibleMelds = findPossibleMelds(hand);
        for (List<Tile> meld : possibleMelds) {
            if (game.isValidMeld(meld)) {
                StringBuilder move = new StringBuilder("P");
                for (Tile tile : meld) {
                    move.append(" ").append(tile.getNumber()).append(" ").append(tile.getColor());
                }
                return move.toString();
            }
        }

        // **Step 3: Try to split a set**
        for (int i = 0; i < game.getBoard().getSets().size(); i++) {
            List<Tile> set = new ArrayList<>(game.getBoard().getSets().get(i));
            if (set.size() > 3) { // Only split sets with more than 3 tiles
                for (int j = 1; j < set.size(); j++) {
                    List<Tile> subset = set.subList(0, j);
                    if (game.isValidMeld(subset)) {
                        return "S " + i + " " + subset.get(0).getNumber() + " " + subset.get(0).getColor(); // Splitting set
                    }
                }
            }
        }

        // **Step 4: Try to merge two sets**
        for (int i = 0; i < game.getBoard().getSets().size(); i++) {
            for (int j = i + 1; j < game.getBoard().getSets().size(); j++) {
                List<Tile> mergedSet = new ArrayList<>(game.getBoard().getSets().get(i));
                mergedSet.addAll(game.getBoard().getSets().get(j));
                if (game.isValidMeld(mergedSet)) {
                    return "J " + i + " " + j; // Merge sets
                }
            }
        }

        // **Step 5: If no play, draw a tile**
        return "DRAW";
    }

    /**
     * Finds possible valid melds that the AI can play from its hand.
     *
     * @param hand The list of tiles in the AI player's hand.
     * @return A list of valid melds that can be played.
     */
    private List<List<Tile>> findPossibleMelds(List<Tile> hand) {
        List<List<Tile>> validMelds = new ArrayList<>();
        Collections.sort(hand, Comparator.comparingInt(Tile::getNumber));

        // **Check for Groups**
        Map<Integer, List<Tile>> groups = new HashMap<>();
        for (Tile tile : hand) {
            if (!tile.isJoker()) {
                groups.putIfAbsent(tile.getNumber(), new ArrayList<>());
                groups.get(tile.getNumber()).add(tile);
            }
        }
        for (List<Tile> group : groups.values()) {
            if (group.size() >= 2) { // If only two tiles, AI can add a Joker
                List<Tile> groupWithJoker = new ArrayList<>(group);
                if (hand.stream().anyMatch(Tile::isJoker)) {
                    groupWithJoker.add(new Tile(0, "Joker", true)); // Add Joker
                }
                if (groupWithJoker.size() >= 3) validMelds.add(groupWithJoker);
            }
        }

        // **Check for Runs**
        Map<String, List<Tile>> runs = new HashMap<>();
        for (Tile tile : hand) {
            if (!tile.isJoker()) {
                runs.putIfAbsent(tile.getColor(), new ArrayList<>());
                runs.get(tile.getColor()).add(tile);
            }
        }
        for (List<Tile> run : runs.values()) {
            Collections.sort(run, Comparator.comparingInt(Tile::getNumber));
            List<Tile> runWithJoker = new ArrayList<>(run);
            if (hand.stream().anyMatch(Tile::isJoker)) {
                runWithJoker.add(new Tile(0, "Joker", true)); // Add Joker
            }
            if (runWithJoker.size() >= 3) validMelds.add(runWithJoker);
        }

        return validMelds;
    }
}
