package MainGame;

import java.util.*;

/**
 * The {@code Game} class represents a tile-based game where players take turns drawing, playing,
 * and manipulating tiles to form valid melds. The game continues until a player wins or
 * the game ends due to insufficient players.
 */

public class Game {
    private Stack<Tile> tilePile;
    private Board board;
    private List<Player> players;
    private Queue<Player> turnQueue;
    private boolean gameStarted;
    private int currentPlayerIndex = 0;

    /**
     * Constructs a new {@code Game} instance with the given list of players.
     *
     * @param players The list of players participating in the game.
     */

    public Game(List<Player> players) {
        this.players = players;
        this.tilePile = new Stack<>();
        this.board = new Board();
        this.turnQueue = new LinkedList<>(players);
        this.gameStarted = false;
    }
    /**
     * Adds a new player to the game.
     *
     * @param player The player to be added.
     */

    public void addPlayer(Player player) {
        players.add(player);
        turnQueue.add(player);
        System.out.println("✅ Player added: " + player.getName() + " (Total Players: " + players.size() + ")");
    }

    /**
     * Initializes the game by generating and shuffling tiles, then distributing 14 tiles to each player.
     */

    public void initializeGame() {
        if (tilePile.isEmpty()) { // Ensure tiles are generated only once
            generateTiles();
            Collections.shuffle(tilePile);
        }

        // Distribute 14 tiles to each player
        for (Player player : players) {
            for (int i = 0; i < 14; i++) {
                if (!tilePile.isEmpty()) {
                    player.addTile(tilePile.pop());
                }
            }
        }
    }
    /**
     * Starts the game, setting the gameStarted flag to true.
     */

    public void startGame() {
        this.gameStarted = true;
        System.out.println("START~Game has started");
    }
    /**
     * Processes a player's move and returns a response indicating the result.
     *
     * @param move   The move command entered by the player.
     * @param player The player making the move.
     * @return A string response indicating the outcome of the move.
     */
    public String processMove(String move, Player player) {
        String[] parts = move.split(" ");

        if (move.equals("DRAW")) { // Drawing a tile
            if (!tilePile.isEmpty()) {
                Tile drawnTile = tilePile.pop();
                player.addTile(drawnTile);
                return "DRAW~" + drawnTile;
            } else {
                return "DRAW~No tiles left";
            }
        }
        if (move.startsWith("P")) { // Playing tiles
            List<Tile> tilesToPlay = parseTilesFromMove(move);

            if (!player.isInitialMeldCompleted()) {
                // ✅ Check if initial meld is valid
                if (!player.canMakeInitialMeld(tilesToPlay)) {
                    return "INVALID~Initial meld must be at least 30 points.";
                }
                player.completeInitialMeld(); // ✅ Mark initial meld as complete
            }

            // ✅ Now allow normal tile placements after the initial meld
            if (!isValidMeld(tilesToPlay)) return "INVALID~Move breaks meld rules";

            for (Tile tile : tilesToPlay) player.removeTile(tile);
            board.addSet(tilesToPlay);

            return "MOVES~Valid move\nUpdated Board: " + board.getSets();
        }

        if (!player.isInitialMeldCompleted() && move.contains("M")) {
            return "INVALID~You cannot modify existing sets until after your initial meld.";
        }

        if (move.startsWith("M")) { // Moving a tile from hand to board
            if (parts.length < 3) return "INVALID~Move command format incorrect";

            Tile tile = parseTile(parts[1] + " " + parts[2]);
            int targetSetIndex;
            try {
                targetSetIndex = Integer.parseInt(parts[3]);
            } catch (NumberFormatException e) {
                return "INVALID~Invalid set index";
            }

            if (targetSetIndex < 0 || targetSetIndex >= board.getSets().size()) {
                return "INVALID~Invalid set reference";
            }

            List<Tile> targetSet = board.getSets().get(targetSetIndex);

            if (!player.getHand().contains(tile)) {
                return "INVALID~Tile not found in player's hand";
            }

            player.removeTile(tile);
            targetSet.add(tile);

            if (!isValidMeld(targetSet)) {
                targetSet.remove(tile);
                player.addTile(tile);
                return "INVALID~Move does not form a valid meld";
            }

            return "MOVES~Tile moved\nUpdated Board: " + board.getSets();
        }

        if (move.startsWith("S")) { // Splitting a set
            if (parts.length < 4) return "INVALID~Split command format incorrect";

            int sourceSetIndex;
            try {
                sourceSetIndex = Integer.parseInt(parts[1]);
            } catch (NumberFormatException e) {
                return "INVALID~Invalid set index";
            }

            if (sourceSetIndex < 0 || sourceSetIndex >= board.getSets().size()) {
                return "INVALID~Invalid set reference";
            }

            List<Tile> sourceSet = new ArrayList<>(board.getSets().get(sourceSetIndex)); // ✅ FIX: Convert to mutable list
            List<Tile> newSet = new ArrayList<>();

            for (int i = 2; i < parts.length; i += 2) { // ✅ FIXED: Read both Number & Color
                if (i + 1 >= parts.length) {
                    return "INVALID~Tile color missing for " + parts[i];
                }

                String tileData = parts[i] + " " + parts[i + 1]; // ✅ FIXED: Combine correctly
                System.out.println("DEBUG: Parsing tile: " + tileData);

                Tile tile = parseTile(tileData);
                if (tile == null) {
                    return "INVALID~Tile parsing failed";
                }

                if (sourceSet.contains(tile)) {
                    sourceSet.remove(tile);
                    newSet.add(tile);
                } else {
                    return "INVALID~Tile not found in source set";
                }
            }

            board.getSets().set(sourceSetIndex, sourceSet); // ✅ FIX: Update the modified list in the board
            board.addSet(newSet);
            return "MOVES~Set split successfully\nUpdated Board: " + board.getSets();
        }


        if (move.startsWith("J")) { // Merging two sets
            if (parts.length < 3) return "INVALID~Merge command format incorrect";

            int set1, set2;
            try {
                set1 = Integer.parseInt(parts[1]);
                set2 = Integer.parseInt(parts[2]);
            } catch (NumberFormatException e) {
                return "INVALID~Invalid set index";
            }

            if (set1 < 0 || set1 >= board.getSets().size() || set2 < 0 || set2 >= board.getSets().size() || set1 == set2) {
                return "INVALID~Invalid set reference";
            }

            List<Tile> mergedSet = new ArrayList<>(board.getSets().get(set1));
            mergedSet.addAll(board.getSets().get(set2));

            if (!isValidMeld(mergedSet)) {
                return "INVALID~Merged set is not a valid meld";
            }

            board.getSets().remove(set2);
            board.getSets().set(set1, mergedSet);

            return "MOVES~Sets merged successfully\nUpdated Board: " + board.getSets();
        }

        if (move.equalsIgnoreCase("EXIT")) {
            System.out.println("DEBUG: Player " + player.getName() + " is exiting the game...");

            players.remove(player); // Remove the player

            // ✅ If only 1 player is left, end the game
            if (players.size() < 2) {
                endGame();
                return "ENDGAME~Game has ended due to insufficient players.";
            }

            return "EXIT~" + player.getName() + " has left the game.";
        }
        return "INVALID~004";
    }

    /**
     * Advances the game to the next player's turn.
     */

    public void nextTurn() {
        do {
            currentPlayerIndex = (currentPlayerIndex + 1) % players.size();
        } while (players.get(currentPlayerIndex) == null); // ✅ Ensure no null players are selected

        System.out.println("🔄 Next Turn: " + players.get(currentPlayerIndex).getName());
    }

    /**
     * Ends the game, cleaning up the board and clearing the player list.
     *
     * @return A string indicating the game has ended.
     */

    private String endGame() {
        board.cleanBoard(); // Reset the board
        players.clear(); // Remove all players
        System.out.println("DEBUG: Game is ending due to insufficient players.");
        return "ENDGAME~Game has ended due to insufficient players.";
    }

    /**
     * Generates and shuffles a full set of game tiles.
     */

    private void generateTiles() {
        tilePile.clear(); // Ensure the tilePile is fresh before generating
        String[] colors = {"R", "B", "Y", "b"};
        for (String color : colors) {
            for (int i = 1; i <= 13; i++) {
                tilePile.push(new Tile(i, color));
                tilePile.push(new Tile(i, color));
            }
        }
        tilePile.push(new Tile(0, "J"));
        tilePile.push(new Tile(0, "J"));
    }

    /**
     * Parses a tile from a given string representation.
     *
     * @param tileData The string representation of the tile.
     * @return The parsed tile object, or null if parsing fails.
     */
    private Tile parseTile(String tileData) {
        System.out.println("DEBUG: Received tile data: " + tileData);

        String[] parts = tileData.trim().split(" ");
        if (parts.length < 2) {
            System.out.println("DEBUG: Invalid tile format: " + tileData);
            return null; // Invalid tile format
        }

        try {
            int number = Integer.parseInt(parts[0]);
            String color = parts[1];
            return new Tile(number, color);
        } catch (NumberFormatException e) {
            System.out.println("DEBUG: Tile parsing failed for " + tileData);
            return null;
        }
    }

    /**
     * Calculates and assigns scores to players based on their remaining tiles.
     *
     * @return A map containing players and their respective scores.
     */
    public Map<Player, Integer> calculateScores() {
        Map<Player, Integer> scores = new HashMap<>();
        Player winner = null;
        int totalPenalty = 0;

        // ✅ **Step 1: Identify the Winner (first player to empty their hand)**
        for (Player player : players) {
            if (player.getHand().isEmpty()) {
                winner = player;
                break;
            }
        }

        // ✅ **Step 2: If No Player Has an Empty Hand, Find Lowest Value Player**
        if (winner == null) {
            int lowestScore = Integer.MAX_VALUE;
            for (Player player : players) {
                int handValue = calculateHandPenalty(player);
                if (handValue < lowestScore) {
                    lowestScore = handValue;
                    winner = player;
                }
            }
        }

        // 🔎 **Debug Output: Check the Winner Before Assigning Scores**
        System.out.println("\n🏆 Winner: " + (winner != null ? winner.getName() : "None"));
        System.out.println("📜 Winner's Hand: " + (winner != null ? winner.getHand() : "None"));

        // ✅ **Step 3: Initialize Scores**
        for (Player player : players) {
            scores.put(player, 0);  // **Ensure all scores start at 0**
        }

        // ✅ **Step 4: Calculate Loser Penalties**
        for (Player player : players) {
            if (player != winner) { // **Only losing players get a penalty**
                int penalty = calculateHandPenalty(player);
                totalPenalty += penalty;  // **Accumulate total penalty**
                scores.put(player, -penalty);  // **Assign negative scores**
            }
        }

        // 🔎 **Debug Output: Total Penalty Before Assigning to Winner**
        System.out.println("⚡ Total Penalty to Assign: " + totalPenalty);

        // ✅ **Step 5: Assign the Winner’s Positive Score AFTER Calculation**
        if (winner != null) {
            scores.put(winner, totalPenalty);  // ✅ **Winner receives total penalties**
        }

        // ✅ **Step 6: Print Final Scores**
        System.out.println("\n📊 Final Scores for the Round:");
        for (Map.Entry<Player, Integer> entry : scores.entrySet()) {
            System.out.println(entry.getKey().getName() + ": " + entry.getValue() + " points");
        }

        return scores;
    }

    /**
     * Calculates the penalty score for a player's remaining tiles.
     *
     * @param player The player whose hand is evaluated.
     * @return The penalty score.
     */

    private int calculateHandPenalty(Player player) {
        int penalty = 0;
        for (Tile tile : player.getHand()) {
            if (tile.isJoker()) {
                penalty += 30; // ✅ Joker penalty is 30 points
            } else {
                penalty += tile.getNumber();
            }
        }
        // 🔎 **Debug: Print each player's hand penalty**
        System.out.println("🔴 " + player.getName() + " Hand Penalty: " + penalty);
        return penalty;
    }

    /**
     * Clears all tiles from the tile pile.
     */
    public void emptyTilePile() {
        this.tilePile.clear(); // ✅ This removes all tiles from the draw pile
        System.out.println("🛑 The tile pile is now empty!");
    }
    /**
     * Checks if any player has won the game.
     *
     * @return {@code true} if a player has won, {@code false} otherwise.
     */
    public boolean checkWinner() {
        for (Player player : players) {
            if (player.hasWon()) {
                System.out.println("ENDGAME~" + player.getName());
                return true;
            }
        }
        return false;
    }
    /**
     * Retrieves the current player.
     *
     * @return The player whose turn it is.
     */
    public Player getCurrentPlayer() {
        Player player = players.get(currentPlayerIndex);
        System.out.println("🕹️ Current Player: " + player.getName() + " (Index: " + currentPlayerIndex + ")");
        return player;
    }
    /**
     * Retrieves the game board.
     *
     * @return The current state of the board.
     */

    public Board getBoard() {
        return board;
    }
    /**
     * Finds a specified tile in a player's hand.
     *
     * @param player The player whose hand to search.
     * @param tile   The tile to find.
     * @return The found tile, or {@code null} if not found.
     */
    public Tile findTileInHand(Player player, Tile tile) {
        for (Tile t : player.getHand()) {
            if (t.isJoker() && tile.isJoker()) {
                System.out.println("DEBUG: Found Joker in Hand!");
                return t; // Return any available Joker
            }
            if (t.equals(tile)) {
                System.out.println("DEBUG: Found Tile in Hand: " + t);
                return t;
            }
        }
        System.out.println("DEBUG: Tile Not Found in Hand: " + tile);
        return null;
    }

    /**
     * Validates whether a set of tiles forms a valid meld.
     *
     * @param set The set of tiles to validate.
     * @return {@code true} if the set is a valid meld, {@code false} otherwise.
     */

    public boolean isValidMeld(List<Tile> set) {
        if (set.size() < 3) return false; // A valid meld needs at least 3 tiles

        boolean isGroup = true;
        boolean isRun = true;
        int firstNumber = -1;
        int jokerCount = 0;
        Set<String> colors = new HashSet<>();
        List<Integer> numbers = new ArrayList<>();

        for (Tile tile : set) {
            if (tile.isJoker()) {
                jokerCount++;
            } else {
                if (firstNumber == -1) firstNumber = tile.getNumber();
                if (tile.getNumber() != firstNumber) isGroup = false; // Different numbers = not a group
                colors.add(tile.getColor());
                numbers.add(tile.getNumber());
            }
        }

        // **Check if Group is Valid**
        if (isGroup) {
            System.out.println("DEBUG: Checking group with Joker count: " + jokerCount);
            return (colors.size() + jokerCount == set.size()); // Jokers can replace missing colors
        }

        // **Check if Run is Valid**
        Collections.sort(numbers);
        for (int i = 1; i < numbers.size(); i++) {
            if (numbers.get(i) != numbers.get(i - 1) + 1) {
                if (jokerCount > 0) {
                    jokerCount--; // Use a Joker to fill the gap
                } else {
                    isRun = false;
                    break;
                }
            }
        }

        return isRun; // Must be a valid run or group
    }

    /**
     * Parses a move command to extract tiles.
     *
     * @param move The move command in the format "P <number> <color> <number> <color> ..."
     * @return A list of Tile objects parsed from the move command.
     */
    private List<Tile> parseTilesFromMove(String move) {
        List<Tile> tiles = new ArrayList<>();
        String[] parts = move.substring(2).trim().split("[, ]+");

        if (parts.length % 2 != 0) return Collections.emptyList(); // Invalid format

        for (int i = 0; i < parts.length; i += 2) {
            try {
                int number = Integer.parseInt(parts[i]);
                String color = parts[i + 1];
                tiles.add(new Tile(number, color));
            } catch (NumberFormatException e) {
                return Collections.emptyList();
            }
        }
        return tiles;
    }



}


