package MainGame;

import java.util.ArrayList;
import java.util.List;

/**
 * The {@code Player} class represents a player in the game.
 * Each player has a hand of tiles, a score, and a status indicating
 * whether they have completed their initial meld.
 */
public class Player {
    private String name;
    private List<Tile> hand;
    private boolean initialMeldCompleted;
    private int score;

    /**
     * Constructs a new {@code Player} with the specified name.
     *
     * @param name The name of the player.
     */
    public Player(String name) {
        this.name = name;
        this.hand = new ArrayList<>();
        this.initialMeldCompleted = true;
        this.score = 0;
    }

    /**
     * Retrieves the player's name.
     *
     * @return The player's name.
     */
    public String getName() { return name; }

    /**
     * Retrieves the player's hand of tiles.
     *
     * @return A list of tiles in the player's hand.
     */
    public List<Tile> getHand() { return hand; }

    /**
     * Checks if the player has completed their initial meld.
     *
     * @return {@code true} if the initial meld is completed, otherwise {@code false}.
     */
    public boolean isInitialMeldCompleted() {

        return initialMeldCompleted;
    }

    /**
     * Retrieves the player's current score.
     *
     * @return The player's score.
     */
    public int getScore() { return score; }

    /**
     * Adds a tile to the player's hand.
     *
     * @param tile The tile to be added.
     */
    public void addTile(Tile tile) { hand.add(tile); }

    /**
     * Removes a tile from the player's hand.
     *
     * @param tile The tile to be removed.
     * @return {@code true} if the tile was successfully removed, otherwise {@code false}.
     */
    public boolean removeTile(Tile tile) { return hand.remove(tile); }

    /**
     * Checks if the player has won the game.
     *
     * @return {@code true} if the player has no tiles left, otherwise {@code false}.
     */
    public boolean hasWon() { return hand.isEmpty(); }

    /**
     * Marks the player's initial meld as completed.
     */
    public void completeInitialMeld() { this.initialMeldCompleted = true; }

    /**
     * Updates the player's score by adding the specified points.
     *
     * @param points The points to be added to the player's score.
     */
    public void updateScore(int points) { this.score += points; }

    /**
     * Parses a string of tile data and adds the parsed tiles to the player's hand.
     *
     * @param tileData A comma-separated string representing tile numbers and colors.
     */
    public void parseTiles(String tileData) {
        String[] tileEntries = tileData.split(",");
        for (String entry : tileEntries) {
            String[] parts = entry.trim().split(" ");
            if (parts.length == 2) {
                try {
                    int number = Integer.parseInt(parts[0]);
                    String color = parts[1];
                    hand.add(new Tile(number, color));
                } catch (NumberFormatException ignored) {}
            }
        }
    }

    /**
     * Calculates the total value of the player's hand.
     *
     * @return The sum of the tile values in the player's hand.
     */
    public int calculateHandValue() {
        int total = 0;
        for (Tile tile : hand) {
            if (tile.isJoker()) {
                total += 30;
            } else {
                total += tile.getNumber();
            }
        }
        return total;
    }
    /**
     * Checks whether the given meld satisfies the initial meld requirement of at least 30 points.
     * Jokers are considered separately and assigned a value based on the tiles they replace.
     *
     * @param meld The list of tiles that the player is attempting to use for the initial meld.
     * @return {@code true} if the total value of the tiles (including Joker values) is at least 30, otherwise {@code false}.
     */

    public boolean canMakeInitialMeld(List<Tile> meld) {
        int totalPoints = 0;
        for (Tile tile : meld) {
            if (tile.isJoker()) {
                continue; // The Joker's value is set separately
            }
            totalPoints += tile.getNumber();
        }

        // Assign Joker values if used
        for (Tile tile : meld) {
            if (tile.isJoker()) {
                int jokerValue = determineJokerValue(meld);
                totalPoints += jokerValue;
            }
        }

        return totalPoints >= 30;
    }

    /**
     * Determines the best possible value for a Joker in the initial meld.
     * The Joker should take the most valuable missing tile in the set.
     */
    private int determineJokerValue(List<Tile> meld) {
        int maxTileValue = 0;
        for (Tile tile : meld) {
            if (!tile.isJoker() && tile.getNumber() > maxTileValue) {
                maxTileValue = tile.getNumber();
            }
        }
        return maxTileValue > 0 ? maxTileValue : 1; // Default Joker value if no valid tile exists
    }
}
