package MainGame;

/**
 * The {@code Tile} class represents a single tile in the game, which has a number and a color.
 * Some tiles can be designated as jokers, which can be used as wildcards in the game.
 */
public class Tile {
    private int number;
    private String color;
    private boolean isJoker;

    /**
     * Constructs a tile with a specified number, color, and joker status.
     *
     * @param number  The number on the tile.
     * @param color   The color of the tile.
     * @param isJoker {@code true} if the tile is a joker, otherwise {@code false}.
     */
    public Tile(int number, String color, boolean isJoker) {
        this.number = number;
        this.color = color;
        this.isJoker = isJoker;
    }

    /**
     * Constructs a tile with a specified number and color.
     * Automatically sets the tile as a joker if the number is 0.
     *
     * @param number The number on the tile.
     * @param color  The color of the tile.
     */
    public Tile(int number, String color) {
        this(number, color, number == 0);
    }

    /**
     * Retrieves the number on the tile.
     *
     * @return The tile's number.
     */
    public int getNumber() {
        return number;
    }

    /**
     * Retrieves the color of the tile.
     *
     * @return The tile's color.
     */
    public String getColor() {
        return color;
    }

    /**
     * Checks if the tile is a joker.
     *
     * @return {@code true} if the tile is a joker, otherwise {@code false}.
     */
    public boolean isJoker() {
        return isJoker;
    }

    /**
     * Returns a string representation of the tile.
     *
     * @return A string representing the tile, showing either "Joker" or its number and color.
     */
    @Override
    public String toString() {
        return (isJoker ? "Joker" : number + color);
    }

    /**
     * Compares this tile with another object for equality.
     * Two tiles are considered equal if they have the same number and color.
     *
     * @param obj The object to compare with this tile.
     * @return {@code true} if the objects are equal, otherwise {@code false}.
     */
    @Override
    public boolean equals(Object obj) {
        if (this == obj) return true;
        if (obj == null || getClass() != obj.getClass()) return false;
        Tile tile = (Tile) obj;
        return number == tile.number && color.equals(tile.color);
    }
}
