package Servers;

import MainGame.Game;
import MainGame.Player;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;
import java.util.List;

/**
 * The {@code Server} class manages the game server, handling client connections and game state updates.
 */
public class Server {
    private int port;
    private Game game;
    private List<ClientHandler> clients;

    /**
     * Constructs a {@code Server} instance with the specified port and game instance.
     *
     * @param port The port on which the server listens for connections.
     * @param game The game instance that manages gameplay logic.
     */
    public Server(int port, Game game) {
        this.port = port;
        this.game = game;
        this.clients = new ArrayList<>();
    }

    /**
     * Starts the server and listens for incoming client connections.
     * Each client connection is handled by a separate thread.
     */
    public void startServer() {
        System.out.println("Server started on port " + port);
        try (ServerSocket serverSocket = new ServerSocket(port)) {
            while (true) {
                Socket clientSocket = serverSocket.accept();
                new Thread(new ClientHandler(clientSocket, this)).start();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * Adds a player to the game and broadcasts the updated game state to all clients.
     *
     * @param player The player to be added to the game.
     */
    public synchronized void addPlayer(Player player) {
        game.addPlayer(player);
        broadcastGameState();
    }

    /**
     * Broadcasts the current game state to all connected clients.
     */
    public synchronized void broadcastGameState() {
        for (ClientHandler client : clients) {
            client.sendGameState();
        }
    }

    /**
     * Retrieves the game instance managed by the server.
     *
     * @return The game instance.
     */
    public Game getGame() {
        return game;
    }
}
