package Tests;
import MainGame.*;
import org.junit.jupiter.api.Test;
import static org.junit.jupiter.api.Assertions.*;
import java.util.*;

/**
 * The {@code RummikubTests} class contains unit tests for various components of the Rummikub game.
 * It tests tile creation, player hand operations, game logic, AI moves, and score calculations.
 */

public class RummikubTests {
    /**
     * Tests the creation of a regular tile.
     */
    @Test
    void testTileCreation() {
        Tile tile = new Tile(5, "Red");
        assertEquals(5, tile.getNumber());
        assertEquals("Red", tile.getColor());
        assertFalse(tile.isJoker());
    }
    /**
     * Tests the creation of a Joker tile.
     */
    @Test
    void testJokerTile() {
        Tile joker = new Tile(0, "Joker", true);
        assertTrue(joker.isJoker());
        assertEquals(0, joker.getNumber());
    }
    /**
     * Tests adding and retrieving tiles from a player's hand.
     */
    @Test
    void testPlayerHandOperations() {
        Player player = new Player("Alice");
        Tile tile = new Tile(3, "Blue");
        player.addTile(tile);
        assertTrue(player.getHand().contains(tile));
    }
    /**
     * Tests validation of a correct group meld.
     */
    @Test
    void testValidMeldGroup() {
        Game game = new Game(new ArrayList<>());
        List<Tile> validGroup = Arrays.asList(
                new Tile(5, "Red"),
                new Tile(5, "Blue"),
                new Tile(5, "Yellow")
        );
        assertTrue(game.isValidMeld(validGroup));
    }
    /**
     * Tests validation of an incorrect group meld.
     */
    @Test
    void testInvalidMeldGroup() {
        Game game = new Game(new ArrayList<>());
        List<Tile> invalidGroup = Arrays.asList(
                new Tile(5, "Red"),
                new Tile(5, "Red"),
                new Tile(6, "Blue")
        );
        assertFalse(game.isValidMeld(invalidGroup));
    }
    /**
     * Tests validation of a correct run meld.
     */
    @Test
    void testValidMeldRun() {
        Game game = new Game(new ArrayList<>());
        List<Tile> validRun = Arrays.asList(
                new Tile(3, "Blue"),
                new Tile(4, "Blue"),
                new Tile(5, "Blue")
        );
        assertTrue(game.isValidMeld(validRun));
    }
    /**
     * Tests validation of an incorrect run meld.
     */
    @Test
    void testInvalidMeldRun() {
        Game game = new Game(new ArrayList<>());
        List<Tile> invalidRun = Arrays.asList(
                new Tile(3, "Blue"),
                new Tile(5, "Blue"),
                new Tile(6, "Blue")
        );
        assertFalse(game.isValidMeld(invalidRun));
    }
    /**
     * Tests AI player's ability to make a valid move.
     */
    @Test
    void testAIPlayerMove() {
        Game game = new Game(new ArrayList<>());
        AIPlayer ai = new AIPlayer("AI_Bot");
        ai.addTile(new Tile(7, "Green"));
        ai.addTile(new Tile(8, "Green"));
        ai.addTile(new Tile(9, "Green"));

        String move = ai.makeMove(game);
        assertTrue(move.startsWith("P")); // AI should play a valid meld
    }
    /**
     * Tests AI's ability to use a Joker in a meld.
     */
    @Test
    void testAIPlayerMoveWithJoker() {
        Game game = new Game(new ArrayList<>());
        AIPlayer ai = new AIPlayer("AI_Bot");
        ai.addTile(new Tile(5, "Red"));
        ai.addTile(new Tile(5, "Blue"));
        ai.addTile(new Tile(0, "Joker", true));

        String move = ai.makeMove(game);
        assertTrue(move.startsWith("P")); // AI should recognize a group with Joker
    }
    /**
     * Tests AI's ability to extend an existing set on the board.
     */
    @Test
    void testAIPlayerMoveToExistingSet() {
        Game game = new Game(new ArrayList<>());
        AIPlayer ai = new AIPlayer("AI_Bot");
        ai.addTile(new Tile(10, "Yellow"));
        ai.addTile(new Tile(11, "Yellow"));
        ai.addTile(new Tile(12, "Yellow"));

        game.getBoard().addSet(Arrays.asList(new Tile(8, "Yellow"), new Tile(9, "Yellow")));
        String move = ai.makeMove(game);
        assertTrue(move.startsWith("M")); // AI should extend an existing run
    }
    /**
     * Tests AI's ability to split a valid set.
     */
    @Test
    void testAIPlayerSplitSet() {
        Game game = new Game(new ArrayList<>());
        AIPlayer ai = new AIPlayer("AI_Bot");
        game.getBoard().addSet(Arrays.asList(
                new Tile(3, "Red"),
                new Tile(4, "Red"),
                new Tile(5, "Red"),
                new Tile(6, "Red")
        ));

        String move = ai.makeMove(game);
        assertTrue(move.startsWith("S")); // AI should recognize a valid split
    }

    /**
     * Tests AI's ability to merge two sets.
     */
    @Test
    void testAIPlayerMergeSets() {
        Game game = new Game(new ArrayList<>());
        AIPlayer ai = new AIPlayer("AI_Bot");
        game.getBoard().addSet(Arrays.asList(new Tile(1, "Blue"), new Tile(2, "Blue")));
        game.getBoard().addSet(Arrays.asList(new Tile(3, "Blue"), new Tile(4, "Blue")));

        String move = ai.makeMove(game);
        assertTrue(move.startsWith("J")); // AI should recognize a valid merge
    }
    /**
     * Tests score calculation after a game round.
     */
    @Test
    void testCalculateScores() {
        List<Player> players = new ArrayList<>();
        Player winner = new Player("Winner");
        Player loser = new Player("Loser");

        winner.addTile(new Tile(5, "Red")); // No penalty, should win
        loser.addTile(new Tile(10, "Blue"));

        players.add(winner);
        players.add(loser);

        Game game = new Game(players);
        Map<Player, Integer> scores = game.calculateScores();

        assertTrue(scores.get(winner) > 0);
        assertTrue(scores.get(loser) < 0);
    }
    /**
     * Tests player attempting to play an invalid meld.
     */
    @Test
    void testPlayerPlayingInvalidMeld() {
        Game game = new Game(new ArrayList<>());
        Player player = new Player("Alice");
        player.addTile(new Tile(6, "Red"));
        player.addTile(new Tile(8, "Red"));
        player.addTile(new Tile(9, "Red"));

        String move = game.processMove("P 6 R 8 R 9 R", player);
        assertTrue(move.startsWith("INVALID")); // Invalid meld, should not be played
    }
    /**
     * Tests player successfully splitting a set.
     */
    @Test
    void testPlayerSplittingSet() {
        Game game = new Game(new ArrayList<>());
        Player player = new Player("Alice");
        game.getBoard().addSet(Arrays.asList(
                new Tile(2, "Y"),
                new Tile(3, "Y"),
                new Tile(4, "Y"),
                new Tile(5, "Y")
        ));

        // ✅ Correct move format: Tiles must be space-separated as "Number Color"
        String move = game.processMove("S 0 2 Y 3 Y 4 Y", player);

        assertTrue(move.startsWith("MOVES"), "Player should successfully split the set");
    }


    /**
     * Tests player moving a tile to an existing set.
     */
    @Test
    void testPlayerMoveTileToExistingSet() {
        Game game = new Game(new ArrayList<>());
        Player player = new Player("Alice");

        Tile tileToMove = new Tile(9, "B");
        player.addTile(tileToMove);

        // ✅ Convert set to a mutable list before adding
        List<Tile> existingSet = new ArrayList<>(Arrays.asList(new Tile(7, "B"), new Tile(8, "B")));
        game.getBoard().addSet(existingSet);

        System.out.println("DEBUG: Before Move -> Board: " + game.getBoard().getSets());
        System.out.println("DEBUG: Before Move -> Player's Hand: " + player.getHand());

        // ✅ Ensure correct move format
        String move = game.processMove("M 9 B 0", player);

        System.out.println("DEBUG: After Move -> Board: " + game.getBoard().getSets());
        System.out.println("DEBUG: After Move -> Player's Hand: " + player.getHand());

        assertTrue(move.startsWith("MOVES"), "Player should successfully extend an existing run");
    }


    /**
     * Tests player playing a valid meld.
     */
    @Test
    void testPlayerPlayingValidMeld() {
        Game game = new Game(new ArrayList<>());
        Player player = new Player("Alice");

        player.addTile(new Tile(6, "R"));
        player.addTile(new Tile(7, "R"));
        player.addTile(new Tile(8, "R"));

        String move = game.processMove("P 6 R 7 R 8 R", player);
        assertTrue(move.startsWith("MOVES"), "Player should successfully play a valid meld");
    }

    /**
     * Tests player merging sets on the board.
     */
    @Test
    void testPlayerMergingSets() {
        Game game = new Game(new ArrayList<>());
        Player player = new Player("Alice");
        game.getBoard().addSet(Arrays.asList(new Tile(1, "Blue"), new Tile(2, "Blue")));
        game.getBoard().addSet(Arrays.asList(new Tile(3, "Blue"), new Tile(4, "Blue")));

        String move = game.processMove("J 0 1", player);
        assertTrue(move.startsWith("MOVES")); // Player should successfully merge two valid sets
    }
    /**
     * Tests that the winner correctly receives the total penalty points of other players.
     */
    @Test
    void testWinnerGetsTotalPenalty() {
        List<Player> players = new ArrayList<>();
        Player winner = new Player("Winner");
        Player loser1 = new Player("Loser 1");
        Player loser2 = new Player("Loser 2");

        // ✅ **Losers have tiles (penalty)**
        loser1.addTile(new Tile(5, "Red"));
        loser1.addTile(new Tile(10, "Blue"));  // Total: 15

        loser2.addTile(new Tile(8, "Yellow"));
        loser2.addTile(new Tile(6, "Blue"));   // Total: 14

        players.add(winner);
        players.add(loser1);
        players.add(loser2);

        Game game = new Game(players);
        Map<Player, Integer> scores = game.calculateScores();

        // ✅ **Winner should receive total penalties (15 + 14 = 29)**
        assertEquals(29, scores.get(winner), "Winner should receive the sum of all penalties");

        // ✅ **Losers should have negative penalties**
        assertEquals(-15, scores.get(loser1), "Loser 1 should have -15 points");
        assertEquals(-14, scores.get(loser2), "Loser 2 should have -14 points");

        // ✅ **Total of all scores must be 0**
        int totalScore = scores.values().stream().mapToInt(Integer::intValue).sum();
        assertEquals(0, totalScore, "Total of all scores should be 0");
    }
    /**
     * Tests that a Joker incurs a 30-point penalty.
     */
    @Test
    void testJokerPenalty() {
        List<Player> players = new ArrayList<>();
        Player winner = new Player("Winner");
        Player loser = new Player("Loser");

        // ✅ **Loser has a Joker (30 penalty)**
        loser.addTile(new Tile(0, "Joker", true));
        loser.addTile(new Tile(7, "Blue"));  // Total: 30 + 7 = 37

        players.add(winner);
        players.add(loser);

        Game game = new Game(players);
        Map<Player, Integer> scores = game.calculateScores();

        assertEquals(-37, scores.get(loser), "Loser should have -37 points (30 for Joker, 7 for tile)");
        assertEquals(37, scores.get(winner), "Winner should get 37 points (sum of penalties)");
    }
    /**
     * Tests tie resolution when the tile pool is empty.
     */
    @Test
    void testTieWhenTilePoolIsEmpty() {
        List<Player> players = new ArrayList<>();
        Player player1 = new Player("Player 1");
        Player player2 = new Player("Player 2");
        Player player3 = new Player("Player 3");

        // ✅ **Simulate the tile pool being empty**
        Game game = new Game(players);
        game.emptyTilePile(); // Assume a method that empties the tile pool

        // ✅ **Player 1 has the lowest total (so they should win)**
        player1.addTile(new Tile(2, "Red")); // Total: 2
        player2.addTile(new Tile(10, "Blue"));
        player2.addTile(new Tile(4, "Yellow")); // Total: 14
        player3.addTile(new Tile(8, "Green"));
        player3.addTile(new Tile(7, "Blue")); // Total: 15

        players.add(player1);
        players.add(player2);
        players.add(player3);

        Map<Player, Integer> scores = game.calculateScores();

        // ❌ Remove incorrect assertion expecting 0 points
        // assertEquals(0, scores.get(player1), "Lowest value player (Player 1) should have 0 penalty");

        // ✅ Corrected expectation: Player 1 receives total penalty sum (14+15=29)
        assertEquals(29, scores.get(player1), "Winner (Player 1) should receive sum of penalties (14+15=29)");

        // ✅ Losers receive negative penalties
        assertEquals(-14, scores.get(player2), "Player 2 should have -14 points");
        assertEquals(-15, scores.get(player3), "Player 3 should have -15 points");

        // ✅ Total score should sum to 0
        int totalScore = scores.values().stream().mapToInt(Integer::intValue).sum();
        assertEquals(0, totalScore, "Total of all scores should be 0");
    }
    /**
     * Tests that the total score of all players always sums to zero.
     */
    @Test
    void testTotalScoreAlwaysZero() {
        List<Player> players = new ArrayList<>();
        Player winner = new Player("Winner");
        Player loser1 = new Player("Loser 1");
        Player loser2 = new Player("Loser 2");

        loser1.addTile(new Tile(3, "Red"));
        loser1.addTile(new Tile(6, "Blue"));  // Total: 9

        loser2.addTile(new Tile(7, "Yellow"));
        loser2.addTile(new Tile(5, "Green"));  // Total: 12

        players.add(winner);
        players.add(loser1);
        players.add(loser2);

        Game game = new Game(players);
        Map<Player, Integer> scores = game.calculateScores();

        // ✅ **Check if all scores add up to 0**
        int totalPoints = scores.values().stream().mapToInt(Integer::intValue).sum();
        assertEquals(0, totalPoints, "Total of all scores should be 0");
    }

    /**
     * Tests that a player cannot play an initial meld if it's worth less than 30 points.
     */
    @Test
    void testInitialMeldRequirement() {
        Game game = new Game(new ArrayList<>());
        Player player = new Player("Alice");

        // Add tiles but total is less than 30
        player.addTile(new Tile(5, "Red"));
        player.addTile(new Tile(7, "Blue"));
        player.addTile(new Tile(8, "Yellow")); // Total: 20 (invalid)

        String move = "P 5 R 7 B 8 Y";
        String response = game.processMove(move, player);

        assertTrue(response.startsWith("INVALID"), "Player should not be allowed to play an initial meld worth less than 30 points");
    }
    /**
     * Tests that a player can successfully play an initial meld if it meets the 30-point requirement.
     */
    @Test
    void testSuccessfulInitialMeld() {
        List<Player> players = new ArrayList<>();
        Player player = new Player("Bob");
        players.add(player);

        Game game = new Game(players);
        game.initializeGame(); // ✅ Make sure the game is properly initialized

        player.addTile(new Tile(10, "R"));
        player.addTile(new Tile(10, "B"));
        player.addTile(new Tile(10, "Y")); // Total: 30 (valid)

        String move = "P 10 R 10 B 10 Y";
        String response = game.processMove(move, player);

        System.out.println("DEBUG: processMove response -> " + response); // ✅ Debug output

        assertTrue(response.startsWith("MOVES"), "Player should be able to play an initial meld worth 30 or more points");
    }


}
